/*
 * ToolbarItem.java
 *
 * Copyright (c) 2009 JAM Development Team
 *
 * This package is distributed under the Lesser Gnu Public Licence (LGPL)
 *
 */

package jam.toolbar;

/**
 * @author rambaut
 *         Date: Oct 18, 2005
 *         Time: 10:19:19 PM
 */
public interface ToolbarItem {
	void setToolbarOptions(ToolbarOptions options);
}
