package jam.panels;

import javax.swing.*;

/**
 * @author rambaut
 *         Date: Oct 31, 2004
 *         Time: 2:47:14 PM
 */
public interface ActionHandler {

	Action getAddAction();
	Action getRemoveAction();
	Action getActionAction();
}
