/*
 * DemoMenuHandler.java
 *
 * Copyright (c) 2009 JAM Development Team
 *
 * This package is distributed under the Lesser Gnu Public Licence (LGPL)
 *
 */

package jam.demo.menus;

import javax.swing.*;

/**
 * @author Andrew Rambaut
 * @version $Id: DemoMenuHandler.java 386 2006-07-17 21:35:49Z rambaut $
 */
public interface DemoMenuHandler {
	Action getFirstAction();
	Action getSecondAction();


}
