/*
 * ControllerListener.java
 *
 * Copyright (c) 2009 JAM Development Team
 *
 * This package is distributed under the Lesser Gnu Public Licence (LGPL)
 *
 */

package jam.controlpalettes;

/**
 * @author Andrew Rambaut
 * @author Alexei Drummond
 * @version $Id: ControllerListener.java 363 2006-06-27 16:26:09Z rambaut $
 */
public interface ControllerListener {
    void controlsChanged();
}
