/*
 * Exportable.java
 *
 * Copyright (c) 2009 JAM Development Team
 *
 * This package is distributed under the Lesser Gnu Public Licence (LGPL)
 *
 */

/**
 * Exportable.java
 */

package jam.framework;

import javax.swing.*;

public interface Exportable {

    JComponent getExportableComponent();
}