/*
 * ControlPaletteListener.java
 *
 * Copyright (c) 2009 JAM Development Team
 *
 * This package is distributed under the Lesser Gnu Public Licence (LGPL)
 *
 */

package jam.controlpalettes;

/**
 * @author Andrew Rambaut
 * @author Alexei Drummond
 * @version $Id: ControlPaletteListener.java 294 2006-04-14 10:28:11Z rambaut $
 */
public interface ControlPaletteListener {
    void controlsChanged();
}
